@@ -682,7 +682,17 @@ typedef uint mrn_srid;
 #endif
 
 #ifdef MRN_MARIADB_P
-#  if (MYSQL_VERSION_ID >= 100203)
+#  if (((MYSQL_VERSION_ID >= 101002) && (MYSQL_VERSION_ID < 101100)) || \
+       ((MYSQL_VERSION_ID >= 100904) && (MYSQL_VERSION_ID < 101000)) || \
+       ((MYSQL_VERSION_ID >= 100806) && (MYSQL_VERSION_ID < 100900)) || \
+       ((MYSQL_VERSION_ID >= 100707) && (MYSQL_VERSION_ID < 100800)) || \
+       ((MYSQL_VERSION_ID >= 100611) && (MYSQL_VERSION_ID < 100700)) || \
+       ((MYSQL_VERSION_ID >= 100518) && (MYSQL_VERSION_ID < 100600)) || \
+       ((MYSQL_VERSION_ID >= 100427) && (MYSQL_VERSION_ID < 100500)) || \
+       ((MYSQL_VERSION_ID >= 100337) && (MYSQL_VERSION_ID < 100400)))
+#    define MRN_GENERATED_COLUMNS_UPDATE_VIRTUAL_FIELD(table, field) \
+       (table->update_virtual_field(field, false))
+#  elif (MYSQL_VERSION_ID >= 100203)
 #    define MRN_GENERATED_COLUMNS_UPDATE_VIRTUAL_FIELD(table, field) \
        (table->update_virtual_field(field))
 #  else
@@ -1052,3 +1062,25 @@ typedef uint mrn_srid;
 #if (MYSQL_VERSION_ID >= 80000 && !defined(MRN_MARIADB_P))
 #  define MRN_HAVE_UDF_METADATA
 #endif
+
+#ifdef MRN_MARIADB_P
+#  define MRN_ERROR_CANCEL ER_STATEMENT_TIMEOUT
+#else
+#  if MYSQL_VERSION_ID >= 50700
+#    define MRN_ERROR_CANCEL ER_QUERY_TIMEOUT
+#  endif
+#endif
+
+#ifdef MRN_ERROR_CANCEL
+#  define MRN_SET_MESSAGE_FROM_CTX(ctx, error_code)     \
+  do {                                                  \
+    if ((ctx)->rc == GRN_CANCEL) {                      \
+      my_error(MRN_ERROR_CANCEL, MYF(0));               \
+    } else {                                            \
+      my_message((error_code), (ctx)->errbuf, MYF(0));  \
+    }                                                   \
+  } while (false)
+#else
+#  define MRN_SET_MESSAGE_FROM_CTX(ctx, error_code)     \
+  my_message(error_code, ctx->errbuf, MYF(0))
+#endif
